
(*
 phazeplot
*)

BeginPackage["phazeplot`"];
Needs["Graphics`PlotField`"];
Needs["Utilities`FilterOptions`"];
ver = "0.1 (23 Apr 2005)";
phasePlot::usage = 
    "phasePlot plots a set of curves in the phase plane\n" <> 
      "phasePlot[equations, {x,xmin, xmax,xdelta}, {y, ymin, ymax,ydelta},{t, tmin, tmax}, options] defines a grid of initial conditions\n" \
<>
      "phasePlot[equations, {x,xmin, xmax}, {y, ymin, ymax},{t, tmin, tmax}, options] defines a grid of initial conditions using the value of the option npoints to determine xdelta and ydelta.\n" \
<>
      "phasePlot[equations, {{x1,y1},{x2,y2},...},{t, tmin, tmax}, options] defines a list of initial conditions\nAny Graphics option or Plot option may be used.\nOther Options:\nfield->False; If true, prints arrows.\nnpoints->25, size of grid spacing if xdelta and ydelta are not specified. ";\

parametricVectorPlot::usage = \
"parametricVectorPlot[equations,{x,y}, t, {xmin,xmax},{ymin,ymax},options] plots the vector field representing the phase portrait.\nx,y, and t are the names of the x,y, and t variables in equations.\n{xmin,xmax},{ymin,ymax} define a grid for the arrows.\nOptions:\nnarrows->5, grid spacing for arrows.";\

parametricPlotVar::usage = "";
plotVar::usage = "";
edgePoints::usage = "";
Options[phasePlot] = {npoints -> 25, field -> False};
npoints::usage = "";
field::usage = "field is an option for phasePlot";
phazer::usage = \
"phazer[equations, {x,xmin,xmax, dx}, {y, ymin,ymax, dy}, {t, tmin,tmax}, options] is similar to phasePlot except that the initial conditions are only computed along the edge of the box defined by {xmin,xmax} by {ymin,ymax} rather than along the entire grid.";\

narrows::usage = "narrows is an option for parametricVectorPlot.";
eigenplot::usage = 
  "eigenplot[A,len,{gdir},options] plots the eigenvectors of the matrix A. The vectors are normalized to length len.\ngdir is a sequence of graphics directives to be applied to the arrow, such as Thickness or Hue.\nAny valid option to Graphis or Graphics`Arrow may be used."

Begin["`Private`"];  
(*
plotVar, parametricPlotVar
plot a variable against a numerical (NDSolve) solution
parametric plot two variables against each other
*)
(*
plotVar[x, n, tfrom, tto, options] plots the variable in the evaluated \
solution over the specified time range
plotVar[x, n, options] plots the entire range of the numerical solution by \
automatically figuring out what it is

plotVarList[n, vlist, options] plots a list of variables on separate plots
plotVarList[n, tfrom, tto, options]

parametricPlotVar[n, {x,y},{t,tfrom,tto},options]
parametricPlotVar[n, {x,y}, t, options]
parametricPlotVar[n, {x,y}, options]
parametricPlotVar[n, {x,y}]

timeRange[n] figures out what the time range of a numerical solution is

The format of n must be like the following example: 
{{a[1][t]->InterpolatingFunction[{{0.,100.}},<>][t],
    a[2][t]->InterpolatingFunction[{{0.,100.}},<>][t],  ...
    y[2][t]->InterpolatingFunction[{{0.,100.}},<>][t],
    z[1][t]->InterpolatingFunction[{{0.,100.}},<>][t],
    z[2][t]->InterpolatingFunction[{{0.,100.}},<>][t]}}


*)
timeRange[n_] := Head[n[[1, 1, 2]]][[1, 1]];
plotVar[x_, n_, opt___?OptionQ] := Module[{tfrom, tto},
    {tfrom, tto} = timeRange[n];
    plotVar[x, n, tfrom, tto, opt]
    ]
plotVar[x_, n_, tfrom_, tto_, opt___?OptionQ] := Module[{popt},
      Return[
        Plot[Evaluate[x[t] /. n], {t, tfrom, tto}, FilterOptions[Plot, opt], 
          PlotLabel -> ToString[x]]]];
plotVarList[n_, tfrom_, tto_, vlist_?ListQ, opt___?OptionQ] := 
    Map[plotVar[#, n, tfrom, tto, opt] &, vlist];
plotVarList[n_, vlist_?ListQ, opt___?OptionQ] := 
    Map[plotVar[#, n, opt] &, vlist];
parametricPlotVar[n_, {x_, y_}, {t_, tfrom_, tto_}, opt___?OptionQ] := 
    
    Module[{popt, args},
      popt = Utilities`FilterOptions`FilterOptions[Plot, opt];
      args = {Evaluate[{x[t], y[t]} /. n], {t, tfrom, tto},
          popt};
      Return[ParametricPlot @@ args
        ]];

parametricPlotVar[n_, {x_, y_}, t, opt___?OptionQ] := 
    Module[{tfrom, tto}, {tfrom, tto} = timeRange[n];
      parametricPlotVar[n, {x, y}, {t, tfrom, tto}, opt]];
parametricPlotVar[n_, {x_, y_}, opt___?OptionQ] := 
    Module[{tfrom, tto, tvariable}, {tfrom, tto} = timeRange[n];
      tvariable = n[[1, 1, 2, 1]];
      parametricPlotVar[n, {x, y}, {tvariable, tfrom, tto}, opt]];
(*
parametricVectorPlot
plot a vector field from a parameterized set of equations
*)
parametricVectorPlot[eqs_, {x_, y_}, t_, xrange_, yrange_, opt___?OptionQ] := 
    Module[{rhs, vecs, xmin, xmax, ymin, ymax, ix, iy, lengthlist, vecvals, 
        veclocs, nlen, vecplot, nvecs},
      
      nvecs = narrows /. {opt} /. {narrows -> 5};
      
      rhs = Map[Last, eqs] /. {x[t] -> ix, y[t] -> iy};
      {xmin, xmax} = xrange;
      {ymin, ymax} = yrange;
      vecs = 
        Flatten[Table[{{ix, iy}, rhs}, {ix, xmin, 
              xmax, (xmax - xmin)/nvecs}, {iy, ymin, 
              ymax, (ymax - ymin)/nvecs}], 1];
      tvecs = Transpose[vecs];
      {veclocs, vecvals} = tvecs;
      lengthlist = Map[Sqrt[Last[#].Last[#]] &, vecs];
      For[i = 1, i <= Length[lengthlist], i++,
         If[lengthlist[[i]] == 0, 
            lengthlist = ReplacePart[lengthlist, 1, i]];
        ];
      vecvals = vecvals/lengthlist;
      nlen = Sqrt[(xmax - xmin)^2 + (ymax - ymin)^2]/25;
      vecvals = vecvals*nlen;
      vecs = Transpose[{veclocs, vecvals}];
      vecplot = ListPlotVectorField[vecs, opt]
      ];
(*
phasePlot
*)
(*
edgePoints
*)
edgePoints[{xmin_, xmax_, dx_}, {ymin_, ymax_, dy_}] := 
  Module[{top, bottom, left, right, v, xl, xr, yb, yt},
    xl = Min[xmin, xmax];
    xr = Max[xmin, xmax];
    yt = Max[ymin, ymax];
    yb = Min[ymin, ymax];
    
    top = Table[{v, yt}, {v, xl, xr, dx}];
    bottom = Table[{v, yb}, {v, xl, xr, dx}];
    left = Table[{xl, v}, {v, yb + dy, yt - dy, dy}];
    right = Table[{xr, v}, {v, yb + dy, yt - dy, dy}];
    
    Return[Join[top, bottom, left, right]]
    
    ]

(*
phasePlot 
*)
\!\(\(\(\[IndentingNewLine]\)\(\[IndentingNewLine]\)\(\(phasePlot[eq_, \ {x_, xmin_, \ xmax_, xdelta_}, \ {y_, \ ymin_, \ ymax_, ydelta_}, {t_, \ tmin_, \ tmax_}, \ opt___?OptionQ] := \ Module[{\ ix, \ iy, ic, n, vars, p, plist = {}, npts, plotfield, vp, gopt, nopt}, \[IndentingNewLine]\[IndentingNewLine]gopt\  = \ FilterOptions[Graphics, opt]; \[IndentingNewLine]nopt = \ FilterOptions[NDSolve, opt]; \[IndentingNewLine]\[IndentingNewLine]vars = {x[t], y[t]}; \[IndentingNewLine]For[ix = xmin, \ ix <= \ xmax, \ ix += \ xdelta, \ \[IndentingNewLine]For[iy = ymin, \ iy <= \ ymax, \ iy += \ ydelta, \ \[IndentingNewLine]\[IndentingNewLine]\[IndentingNewLine]ic = {x[tmin] == \ ix, \ y[tmin] == \ iy}; \[IndentingNewLine]\[IndentingNewLine]n = NDSolve[Join[eq, ic], vars, {t, tmin, tmax}, nopt]; \[IndentingNewLine]p = parametricPlotVar[n, {x, y}, {t, tmin, tmax}, opt, PlotRange -> \ All, \ DisplayFunction -> \ Identity]; \[IndentingNewLine]plist\  = \ Append[plist, p];\[IndentingNewLine]\[IndentingNewLine]]\[IndentingNewLine]]; \[IndentingNewLine]plotfield\  = \ \(field /. {opt}\) /. Options[phasePlot]; \[IndentingNewLine]If[plotfield, \ \[IndentingNewLine]vp = parametricVectorPlot[eq, {x, y}, t, {xmin, xmax}, {ymin, ymax}, opt, DisplayFunction -> \ Identity]; \[IndentingNewLine]Return[Show[plist, vp, gopt, \ DisplayFunction -> \ $DisplayFunction, \ PlotRange -> \ All]]\[IndentingNewLine]]; \[IndentingNewLine]\[IndentingNewLine]Show[plist, gopt, DisplayFunction -> \ $DisplayFunction, \ PlotRange -> \ All]\[IndentingNewLine]];\)\[IndentingNewLine]
  \(phasePlot[eq_, \ {x_, xmin_, \ xmax_}, \ {y_, \ ymin_, \ ymax_}, {t_, \ tmin_, \ tmax_}, \ opt___?OptionQ] := \ Module[{nx, ny, np, xdelta, ydelta}, \[IndentingNewLine]np\  = \ \(npoints /. {opt}\) /. Options[phasePlot]; \[IndentingNewLine]\[IndentingNewLine]nx\  = \ Ceiling[\@np]; \[IndentingNewLine]ny = nx; \[IndentingNewLine]xdelta\  = \ \((xmax - xmin)\)/nx; \[IndentingNewLine]ydelta\  = \ \((ymax - ymin)\)/ny; \[IndentingNewLine]\[IndentingNewLine]phasePlot[eq, \ {x, xmin, \ xmax, xdelta}, \ {y, \ ymin, \ ymax, ydelta}, {t, \ tmin, \ tmax}, \ opt]\[IndentingNewLine]];\)\[IndentingNewLine]\[IndentingNewLine]
  \(phasePlot[eq_, \ {pt__?ListQ}, {x_, y_}, \ {t_, \ tmin_, \ tmax_}, \ opt___?OptionQ] := \ Module[{\ ix, \ iy, ic, n, vars, p, plist = {}, npts, plotfield, vp, gopt, \ pts, \ nopt}, \[IndentingNewLine]\[IndentingNewLine]npts = Union[Length /@ {pt}]; \[IndentingNewLine]If[Length[Complement[npts, {2}]] > 0, \ \[IndentingNewLine]Print["\<Invalid list of points, \>", \ {pt}]; \[IndentingNewLine]Abort[];\[IndentingNewLine]]; \[IndentingNewLine]\[IndentingNewLine]gopt\  = \ FilterOptions[Graphics, opt]; \[IndentingNewLine]nopt = \ FilterOptions[NDSolve, opt]; \[IndentingNewLine]\[IndentingNewLine]vars = {x[t], y[t]}; \[IndentingNewLine]pts = {pt}; \[IndentingNewLine]While[Length[pts] > 0, \ \[IndentingNewLine]{ix, iy} = First[pts]; \[IndentingNewLine]pts = Rest[pts]; \[IndentingNewLine]\[IndentingNewLine]ic = {x[tmin] == \ ix, \ y[tmin] == \ iy}; \[IndentingNewLine]\[IndentingNewLine]n = NDSolve[Join[eq, ic], vars, {t, tmin, tmax}, nopt]; \[IndentingNewLine]\[IndentingNewLine]p = parametricPlotVar[n, {x, y}, {t, tmin, tmax}, opt, PlotRange -> \ All, \ DisplayFunction -> \ Identity]; \[IndentingNewLine]plist\  = \ Append[plist, p];\[IndentingNewLine]\[IndentingNewLine]\[IndentingNewLine]]; \[IndentingNewLine]\[IndentingNewLine]\[IndentingNewLine]Show[plist, gopt, DisplayFunction -> \ $DisplayFunction, \ PlotRange -> \ All]\[IndentingNewLine]];\)\[IndentingNewLine]\[IndentingNewLine]
  \)\)\)
(*
phazer
*)
phazer[eq_, {x_, xmin_, xmax_, dx_}, {y_, ymin_, ymax_, dy_}, {t_, tmin_, 
      tmax_}, opt___?OptionQ] := Module[{p1, p2, edge, gopt},
    edge = edgePoints[{xmin, xmax, dx}, {ymin, ymax, dy}];
    p1 = phasePlot[eq, edge, {x, y}, {t, tmin, tmax}, 
        DisplayFunction -> Identity, opt];
    gopt = FilterOptions[Graphics, opt];
    If[field /. {opt},
      p2 = 
        parametricVectorPlot[eq, {x, y}, t, {xmin, xmax}, {ymin, ymax}, 
          DisplayFunction -> Identity, opt, 
          PlotRange -> {{xmin, xmax}, {ymin, ymax}}
          ];
      Return[Show[{p1, p2}, gopt, DisplayFunction -> $DisplayFunction]];
      ];
    Return[Show[p1, gopt, DisplayFunction -> $DisplayFunction]];
    ]
(*
eigenplot
*)
\!\(eigenplot[M_, \ l_, {gdir___}, opt___?OptionQ] := \ Module[{v1, v2, aopt, gopt, g}, \[IndentingNewLine]aopt = FilterOptions[Arrow, opt]; \[IndentingNewLine]{v1, v2} = Eigenvectors[M]; \[IndentingNewLine]{v1, v2} = Map[l*#/\@\(# . #\) &, {v1, v2}]; \[IndentingNewLine]{v1, v2} = \(Arrow[{0, 0}, #, aopt] &\) /@ {v1, v2}; \[IndentingNewLine]gopt = FilterOptions[Graphics, opt]; \[IndentingNewLine]Show[{\(Graphics[{gdir, #}] &\) /@ {v1, v2}} // Flatten, gopt]\[IndentingNewLine]]\)
Print["phazeplot ", ver, " loaded."]; 
End[];
EndPackage[];
